package java.awt.image;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import sun.awt.image.ImagingLib;
import java.awt.RenderingHints;

public class RescaleOp implements BufferedImageOp, RasterOp
{
    float[] scaleFactors;
    float[] offsets;
    int length;
    RenderingHints hints;
    private int srcNbits;
    private int dstNbits;
    
    public RescaleOp(float[] array, float[] array2, RenderingHints hints) {
        this.length = 0;
        this.length = array.length;
        if (this.length > array2.length) {
            this.length = array2.length;
        }
        this.scaleFactors = new float[this.length];
        this.offsets = new float[this.length];
        for (int i = 0; i < this.length; ++i) {
            this.scaleFactors[i] = array[i];
            this.offsets[i] = array2[i];
        }
        this.hints = hints;
    }
    
    public RescaleOp(float n, float n2, RenderingHints hints) {
        this.length = 0;
        this.length = 1;
        this.scaleFactors = new float[1];
        this.offsets = new float[1];
        this.scaleFactors[0] = n;
        this.offsets[0] = n2;
        this.hints = hints;
    }
    
    public final float[] getScaleFactors(float[] array) {
        if (array == null) {
            return (float[])(float[])this.scaleFactors.clone();
        }
        System.arraycopy(this.scaleFactors, 0, array, 0, Math.min(this.scaleFactors.length, array.length));
        return array;
    }
    
    public final float[] getOffsets(float[] array) {
        if (array == null) {
            return (float[])(float[])this.offsets.clone();
        }
        System.arraycopy(this.offsets, 0, array, 0, Math.min(this.offsets.length, array.length));
        return array;
    }
    
    public final int getNumFactors() {
        return this.length;
    }
    
    private ByteLookupTable createByteLut(float[] array, float[] array2, int n, int n2) {
        byte[][] array3 = new byte[array.length][n2];
        for (int i = 0; i < array.length; ++i) {
            float n3 = array[i];
            float n4 = array2[i];
            byte[] array4 = array3[i];
            int n5;
            for (int j = 0; j < n2; array4[j] = (byte)n5, ++j) {
                n5 = (int)((float)j * n3 + n4);
                if ((n5 & 0xFFFFFF00) != 0x0) {
                    if (n5 < 0) {
                        n5 = 0;
                    }
                    else {
                        n5 = 255;
                    }
                }
            }
        }
        return new ByteLookupTable(0, array3);
    }
    
    private ShortLookupTable createShortLut(float[] array, float[] array2, int n, int n2) {
        short[][] array3 = new short[array.length][n2];
        for (int i = 0; i < array.length; ++i) {
            float n3 = array[i];
            float n4 = array2[i];
            short[] array4 = array3[i];
            int n5;
            for (int j = 0; j < n2; array4[j] = (short)n5, ++j) {
                n5 = (int)((float)j * n3 + n4);
                if ((n5 & 0xFFFF0000) != 0x0) {
                    if (n5 < 0) {
                        n5 = 0;
                    }
                    else {
                        n5 = 65535;
                    }
                }
            }
        }
        return new ShortLookupTable(0, array3);
    }
    
    private boolean canUseLookup(Raster raster, Raster raster2) {
        int dataType = raster.getDataBuffer().getDataType();
        if (dataType != 0 && dataType != 1) {
            return false;
        }
        SampleModel sampleModel = raster2.getSampleModel();
        this.dstNbits = sampleModel.getSampleSize(0);
        if (this.dstNbits != 8 && this.dstNbits != 16) {
            return false;
        }
        int i = 1;
        while (i < raster.getNumBands()) {
            if (sampleModel.getSampleSize(i) != this.dstNbits) {
                return false;
            }
            ++i;
        }
        SampleModel sampleModel2 = raster.getSampleModel();
        this.srcNbits = sampleModel2.getSampleSize(0);
        if (this.srcNbits > 16) {
            return false;
        }
        int j = 1;
        while (j < raster.getNumBands()) {
            if (sampleModel2.getSampleSize(j) != this.srcNbits) {
                return false;
            }
            ++j;
        }
        return true;
    }
    
    @Override
    public final BufferedImage filter(BufferedImage src, BufferedImage dst) {
        ColorModel colorModel = src.getColorModel();
        int numColorComponents = colorModel.getNumColorComponents();
        if ((colorModel instanceof IndexColorModel)) {
            throw new IllegalArgumentException("Rescaling cannot be performed on an indexed image");
        }
        if (this.length != 1 && this.length != numColorComponents && this.length != colorModel.getNumComponents()) {
            throw new IllegalArgumentException("Number of scaling constants does not equal the number of of color or color/alpha  components");
        }
        boolean n = false;
        if (this.length > numColorComponents && colorModel.hasAlpha()) {
            this.length = numColorComponents + 1;
        }
        int width = src.getWidth();
        int height = src.getHeight();
        if (dst == null) {
            dst = this.createCompatibleDestImage(src, null);
        }
        else {
            if (width != dst.getWidth()) {
                throw new IllegalArgumentException("Src width (" + width + ") not equal to dst width (" + dst.getWidth() + ")");
            }
            if (height != dst.getHeight()) {
                throw new IllegalArgumentException("Src height (" + height + ") not equal to dst height (" + dst.getHeight() + ")");
            }
            colorModel = dst.getColorModel();
            if (colorModel.getColorSpace().getType() != colorModel.getColorSpace().getType()) {
                n = true;
                dst = this.createCompatibleDestImage(src, null);
            }
        }
        if (ImagingLib.filter(this, src, dst) == null) {
            WritableRaster writableRaster = src.getRaster();
            WritableRaster writableRaster2 = dst.getRaster();
            if (colorModel.hasAlpha() && (numColorComponents - 1 == this.length || this.length == 1)) {
                int minX = writableRaster.getMinX();
                int minY = writableRaster.getMinY();
                int[] array = new int[numColorComponents - 1];
                for (int i = 0; i < numColorComponents - 1; ++i) {
                    array[i] = i;
                }
                writableRaster = writableRaster.createWritableChild(minX, minY, writableRaster.getWidth(), writableRaster.getHeight(), minX, minY, array);
            }
            if (colorModel.hasAlpha() && (writableRaster2.getNumBands() - 1 == this.length || this.length == 1)) {
                int minX2 = writableRaster2.getMinX();
                int minY2 = writableRaster2.getMinY();
                int[] array2 = new int[numColorComponents - 1];
                for (int j = 0; j < numColorComponents - 1; ++j) {
                    array2[j] = j;
                }
                writableRaster2 = writableRaster2.createWritableChild(minX2, minY2, writableRaster2.getWidth(), writableRaster2.getHeight(), minX2, minY2, array2);
            }
            this.filter(writableRaster, writableRaster2);
        }
        if (!(n == 0)) {
            new ColorConvertOp(this.hints).filter(dst, dst);
            return dst;
        }
        return dst;
    }
    
    @Override
    public final WritableRaster filter(Raster src, WritableRaster compatibleDestRaster) {
        int numBands = src.getNumBands();
        int width = src.getWidth();
        int height = src.getHeight();
        int[] pixel = null;
        int n = 0;
        if (compatibleDestRaster == null) {
            compatibleDestRaster = this.createCompatibleDestRaster(src);
        }
        else {
            if (height != compatibleDestRaster.getHeight() || width != compatibleDestRaster.getWidth()) {
                throw new IllegalArgumentException("Width or height of Rasters do not match");
            }
            if (numBands != compatibleDestRaster.getNumBands()) {
                throw new IllegalArgumentException("Number of bands in src " + numBands + " does not equal number of bands in dest " + compatibleDestRaster.getNumBands());
            }
        }
        if (this.length != 1 && this.length != src.getNumBands()) {
            throw new IllegalArgumentException("Number of scaling constants does not equal the number of of bands in the src raster");
        }
        if (ImagingLib.filter(this, src, compatibleDestRaster) != null) {
            return compatibleDestRaster;
        }
        if (this.canUseLookup(src, compatibleDestRaster)) {
            int n4 = 1 << this.srcNbits;
            if (1 << this.dstNbits == 256) {
                new LookupOp(this.createByteLut(this.scaleFactors, this.offsets, numBands, n4), this.hints).filter(src, compatibleDestRaster);
            }
            else {
                new LookupOp(this.createShortLut(this.scaleFactors, this.offsets, numBands, n4), this.hints).filter(src, compatibleDestRaster);
            }
            return compatibleDestRaster;
        }
        else {
            if (this.length > 1) {
                n = 1;
            }
            int minX = src.getMinX();
            int minY = src.getMinY();
            int minX2 = compatibleDestRaster.getMinX();
            int minY2 = compatibleDestRaster.getMinY();
            int[] array = new int[numBands];
            int[] array2 = new int[numBands];
            SampleModel sampleModel = compatibleDestRaster.getSampleModel();
            for (int i = 0; i < numBands; ++i) {
                array[i] = (1 << sampleModel.getSampleSize(i)) - 1;
                array2[i] = ~array[i];
            }
            for (int j = 0; j < height; ++j, ++minY, ++minY2) {
                for (int k = 0; k < width; ++k, ++minX, ++minX2) {
                    pixel = src.getPixel(minX, minY, pixel);
                    int n3;
                    for (int n2 = 0, l = 0; l < numBands; pixel[l] = n3, ++l, n2 += n) {
                        n3 = (int)((float)pixel[l] * this.scaleFactors[n2] + this.offsets[n2]);
                        if ((n3 & array2[l]) != 0x0) {
                            if (n3 < 0) {
                                n3 = 0;
                            }
                            else {
                                n3 = array[l];
                            }
                        }
                    }
                    compatibleDestRaster.setPixel(minX2, minY2, pixel);
                }
            }
            return compatibleDestRaster;
        }
    }
    
    @Override
    public final Rectangle2D getBounds2D(BufferedImage bufferedImage) {
        return this.getBounds2D(bufferedImage.getRaster());
    }
    
    @Override
    public final Rectangle2D getBounds2D(Raster raster) {
        return raster.getBounds();
    }
    
    @Override
    public BufferedImage createCompatibleDestImage(BufferedImage bufferedImage, ColorModel colorModel) {
        if (colorModel == null) {
            ColorModel colorModel2 = bufferedImage.getColorModel();
            return new BufferedImage(colorModel2, bufferedImage.getRaster().createCompatibleWritableRaster(), colorModel2.isAlphaPremultiplied(), null);
        }
        return new BufferedImage(colorModel, colorModel.createCompatibleWritableRaster(bufferedImage.getWidth(), bufferedImage.getHeight()), colorModel.isAlphaPremultiplied(), null);
    }
    
    @Override
    public WritableRaster createCompatibleDestRaster(Raster raster) {
        return raster.createCompatibleWritableRaster(raster.getWidth(), raster.getHeight());
    }
    
    @Override
    public final Point2D getPoint2D(Point2D point2D, Point2D point2D2) {
        if (point2D2 == null) {
            point2D2 = new Point2D.Float();
        }
        point2D2.setLocation(point2D.getX(), point2D.getY());
        return point2D2;
    }
    
    @Override
    public final RenderingHints getRenderingHints() {
        return this.hints;
    }
}
